import configparser
from typing import Type
from yacmmal.load.base import Loader
from pydantic import BaseModel

class CONFLoader(Loader):
    """
    Loader for CONF files.

    Attributes
    ----------
    base_path : str
        The base path of the config files.
    """

    def __init__(self, base_path: str):
        super(CONFLoader, self).__init__(base_path=base_path)
        self.format = "conf"

    def load(self, path: str, dclass: Type[BaseModel]) -> BaseModel:
        """
        Loads a TOML file as a dataclass.

        Parameters
        ----------
        path : str
            The path to the CONF file.
        dclass : Type[BaseModel]
            The dataclass to load the CONF file as.

        Returns
        -------
        BaseModel
            The loaded dataclass.
        """
        config = configparser.ConfigParser()
        config.read(path)

        data = {section: dict(config[section]) for section in config.sections()}
        return dclass.parse_obj(data)

class INILoader(CONFLoader):
    """
    Loader for INI files.

    Attributes
    ----------
    base_path : str
        The base path of the config files.
    """
    def __init__(self, base_path: str):
        super(INILoader, self).__init__(base_path=base_path)
        self.format = "ini"

    def load(self, path: str, dclass: Type[BaseModel]) -> BaseModel:
        """
        Loads a INI file as a dataclass.

        Parameters
        ----------
        path : str
            The path to the INI file.
        dclass : Type[BaseModel]
            The dataclass to load the INI file as.
        """
        return super(INILoader, self).load(path=path, dclass=dclass)



from pydantic import BaseModel    
from typing import Optional
from enum import Enum, auto

class Config(BaseModel):
    """
    The default configuration dataclass of yacmmal.
    """
    paths: Optional[BaseModel]
    database: Optional[BaseModel]
    hyperparameters: Optional[BaseModel]
    experiment: Optional[BaseModel]
    training: Optional[BaseModel]
    evaluation: Optional[BaseModel]
    optimization: Optional[BaseModel]
    deploy: Optional[BaseModel]

class ConfigAttrs(Enum):
    """
    The attributes supported by yacmmal's Config dataclass.
    """
    paths = auto()
    database = auto()
    hyperparameters = auto()
    experiment = auto()
    training = auto()
    evaluation = auto()
    optimization = auto()
    deploy = auto()



import os
from abc import ABC, abstractmethod
from yacmmal.types.config import Config, ConfigAttrs
from pydantic import BaseModel
from typing import Dict, Type, Union

class AbstractLoader(ABC):
    """
    Abstract class for config loaders.
    """
    @abstractmethod
    def add_path(
        self, path: str, name: str, dclass: Type[BaseModel]
        ) -> "AbstractLoader":
        ...

    @abstractmethod
    def load(self, path: str, dclass: Type[BaseModel]) -> BaseModel:
        ...

    @abstractmethod
    def extract(self) -> Config:
        ...

class Loader(AbstractLoader):
    """
    Base class for loading config files.
    """
    def __init__(self, base_path: str):
        self.data: Dict[str, BaseModel] = {}
        self.format: str = ""
        self.base_path: str = base_path

    def add_path(self, path: str, name: Union[str, ConfigAttrs], dclass: Type[BaseModel]) -> "Loader":
        """
        Adds a path to the loader.

        Parameters
        ----------
        path : str
            Path to the config file.
        name : Union[str, ConfigAttrs]
            Name of the config file.
        dclass : Type[BaseModel]
            The dataclass to use.

        Returns
        -------
        Loader
            The current loader.
        """
        if isinstance(name, str):
            name = ConfigAttrs[name]
        file_name = ".".join([path, self.format])
        file_path = os.path.join(self.base_path, file_name)
        self.data[name.name] = self.load(file_path, dclass)
        return self

    def extract(self) -> Config:
        """
        Extracts the config object.

        Returns
        -------
        Config
            The config object.
        """
        return Config(**self.data)



import toml
from typing import Type
from yacmmal.load.base import Loader
from pydantic import BaseModel

class TOMLLoader(Loader):
    """
    Loader for TOML files.

    Attributes
    ----------
    base_path : str
        The base path of the config files.
    """

    def __init__(self, base_path: str):
        super(TOMLLoader, self).__init__(base_path=base_path)
        self.format = "toml"

    def load(self, path: str, dclass: Type[BaseModel]) -> BaseModel:
        """
        Loads a TOML file as a dataclass.

        Parameters
        ----------
        path : str
            The path to the TOML file.
        dclass : Type[BaseModel]
            The dataclass to load the TOML file as.

        Returns
        -------
        BaseModel
            The loaded dataclass.
        """
        with open(path, "r") as json_file:
            data = toml.load(json_file)
        return dclass.parse_obj(data)



from enum import Enum, auto

class ConfigFormat(Enum):
    """
    Enum for the different formats of the data.
    """
    JSON = auto()
    YAML = auto()
    TOML = auto()
    CONF = auto()
    INI = auto()
